﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace LSystems
{
    public class System
    {
        private List<ProductionRule> rules;
        private List<object> currentString;
        private object systemDefinition;

        internal System(List<ProductionRule> rules, object systemDefinition)
        {
            this.systemDefinition = systemDefinition;
            this.rules = rules;
            this.currentString = new List<object>();
        }

        public object String
        {
            get
            {
                return this.currentString;
            }

            set
            {
                this.currentString = value.AsList();
            }
        }

        public void RewriteLeftToRight()
        {
            List<object> newString = new List<object>();

            for (int i = 0; i < this.currentString.Count; ++i)
            {
                foreach (var rule in this.rules)
                {
                    object replacement = rule.TryApply(currentString, i, newString, null, systemDefinition);
                    if (replacement is EmptyModule)
                    {
                        // Do nothing, strict is removed from new string.
                        goto endOfReplacement;
                    }
                    else if (replacement != null)
                    {
                        // Have some replacement.
                        newString.InsertRange(newString.Count, replacement.AsList());
                        i += rule.strict.Count - 1;
                        goto endOfReplacement;
                    }                    
                }

                // No replacement were found, 
                // copy old module to the new string.
                newString.Add(this.currentString[i]);

            endOfReplacement: ;
            }

            this.currentString = newString;
        }        

        public void RewriteRightToLeft()
        {
            List<object> newString = new List<object>();

            for (int i = this.currentString.Count - 1; i >= 0; --i)
            {
                foreach (var rule in this.rules)
                {
                    object replacement = rule.TryApply(this.currentString, i, null, newString, this.systemDefinition);
                    if (replacement is EmptyModule)
                    {
                        // Do nothing, strict is removed from new string.
                        goto endOfReplacement;
                    }
                    else if (replacement != null)
                    {
                        // Have some replacement.
                        newString.InsertRange(0, replacement.AsList());
                        i -= rule.strict.Count - 1;
                        goto endOfReplacement;
                    }
                }

                // No replacement were found, 
                // copy old module to the new string.
                newString.Insert(0, this.currentString[i]);

            endOfReplacement: ;
            }

            this.currentString = newString;
        }        
    }
}
