﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.ComponentModel;

namespace Test
{
    public class W
    {
        public double T { get; private set; }
        public W(double t)
        {
            this.T = t;
        }
        public override string ToString()
        {
            return string.Format("W({0:0.00})", this.T);
        }
    }

    public class M
    {
        public double T { get; private set; }
        public M(double t)
        {
            this.T = t;
        }
        public override string ToString()
        {
            return string.Format("M({0:0.00})", this.T);
        }
    }

    public class AnabaenaDefinition
    {
        public const double Div = 1;
        public const double ShortCell = 0;
        public const double LongCell = 0.2;
        public const double TimeStep = 0.7;
        public const double Epsilon = 0.000001;

        [LSystems.Production("w")]
        public object WallGrow(W w)
        {
            return new W(w.T + TimeStep);
        }

        [LSystems.Production("m")]
        public object CellGrow(M m)
        {
            if (m.T < Div - Epsilon)
                return new M(m.T + TimeStep);
            else
                return null;
        }

        [LSystems.Production("leftWall < m > rightWall")]
        public object Divide(W leftWall, M m, W rightWall)
        {
            if (m.T > Div - Epsilon)
            {
                if (leftWall.T < rightWall.T)
                {
                    return new object[] 
                        {
                            new M(m.T - Div + LongCell),
                            new W(m.T - Div),
                            new M(m.T - Div + ShortCell),
                        };
                }
                else
                {
                    return new object[] 
                        {
                            new M(m.T - Div + ShortCell),
                            new W(m.T - Div),
                            new M(m.T - Div + LongCell),
                        };
                }
            }
            return null;
        }
    }

    class Program
    {        
        static void Main(string[] args)        
        {
            // Create L-System.
            var definition = new AnabaenaDefinition();
            LSystems.System system = 
                LSystems.SystemBuilder.BuildSystem(definition);
            system.String = new object[] 
            {
                new W(0), new M(0), new W(AnabaenaDefinition.TimeStep)
            };
            
            // Rewrite and printout the result.
            Print(system.String as IEnumerable);

            for (int i = 0; i < 7; ++i)
            {                
                system.RewriteLeftToRight();
                Print(system.String as IEnumerable);
            }

            Console.WriteLine("Press any key to exit...");
            Console.ReadKey();
        }

        private static void Print(IEnumerable modules)
        {
            foreach (object module in modules)
            {
                Console.Write("{0} ", module);
            }
            Console.WriteLine();
        }
    }
}
